﻿-- print("Modules/Initialise.lua start")
-- *********************************
-- * Variable/Constant Declaration *
-- *********************************

LFMonitor.settingsVersion = "50205" -- used for forcing old settings to reset
LFMonitor.cacheVersion = "50207" -- used for wiping the cache
LFMonitor.defaultCache = {count=0, max = 8192, version = LFMonitor.cacheVersion}
LFMonitor.LFM_Update_Int = 20.0
LFMonitor.CL_Update_Int = 0.5
LFMonitor.testmode = false
LFMonitor.history = {}
LFMonitor.hindex = 1
LFMonitor.TFW_PATTERN = "^_$(%d+)(.+)_$(.+)_$(.+)%$$"
LFMonitor.thisRealm = GetRealmName()

-- function to deepcopy a table, from http://lua-users.org/wiki/CopyTable
function LFMonitor:deepcopy(object)

    local lookup_table = {}
    local function _copy(object)

        if type(object) ~= "table" then

            return object

        elseif lookup_table[object] then

            return lookup_table[object]

        end

        local new_table = {}
        lookup_table[object] = new_table

        for index, value in pairs(object) do

            new_table[_copy(index)] = _copy(value)

        end

        return setmetatable(new_table, getmetatable(object))

    end

    return _copy(object)

end

if not LFMonitorCache then
	LFMonitorCache = LFMonitor:deepcopy(LFMonitor.defaultCache)
end
if not LFMonitorWordCache then
	LFMonitorWordCache = LFMonitor:deepcopy(LFMonitor.defaultCache)
end
if not LFMonitorClassCacheName then
	LFMonitorClassCacheName = {[LFMonitor.thisRealm] = LFMonitor:deepcopy(LFMonitor.defaultCache)}
	LFMonitorClassCacheGUID = {[LFMonitor.thisRealm] = LFMonitor:deepcopy(LFMonitor.defaultCache)}
elseif not LFMonitorClassCacheName[LFMonitor.thisRealm] then
	LFMonitorClassCacheName[LFMonitor.thisRealm] = LFMonitor:deepcopy(LFMonitor.defaultCache)
	LFMonitorClassCacheGUID[LFMonitor.thisRealm] = LFMonitor:deepcopy(LFMonitor.defaultCache)
end
--concatenate two tables to produce one table
local function concatenate(t1, t2)

	local ctable = LFMonitor:deepcopy(t1)

	for _, txt in ipairs(t2) do

		table.insert(ctable, txt)

	end

	return ctable

end

--merge two or more dictionaries to produce one dictionary
local function dic_concat(...)
	local out = {}
	local i, table
	local n=select('#',...)
	for i=1,n do
		table = select(i,...)
		for key, value in pairs(table) do
			if out[key] then
				out[key] = math.max(out[key], value)
			else
				out[key] = value
			end
		end
	end
	return out
end

--build search tables
function LFMonitor:BuildTables()
	-- this function doesn't do anything anymore
end

LFMonitor.patterns = {
	["<.*>"] = {
		["type"] = {
			["guildlfm"] = 1
		},
	},
	["^%s*%[.*%]"] = {
		["type"] = {
			["guildlfm"] = 1
		},
	},
	["Hitem"] = {
		["type"] = {
			["wts"] = 0.1,
			["wtb"] = 0.75
		},
	},
	["Hachievement"] = {
		["type"] = {
			["lfg"] = 0.75,
			["lfm"] = 0.75,
			["lfguild"] = 0.5,
		},
	},
	["Hinstancelock"] = {
		["type"] = {
			["lfg"] = 0.75,
			["lfm"] = 0.75,
		},
	},
	["Hquest"] = {
		["type"] = {
			["lfg"] = 0.75,
			["lfm"] = 0.75,
		},
	},
}

LFMonitor.sounds = {
	[1] = {
		file = "LOOTWINDOWCOINSOUND",
		type = "game"
	},
	[2] = {
		file = "INTERFACESOUND_CHARWINDOWTAB",
		type = "game"
	},
	[3] = {
		file = "INTERFACESOUND_GAMEMENUOPEN",
		type = "game"
	},
	[4] = {
		file = "igMiniMapOpen",
		type = "game"
	},
	[5] = {
		file = "igSpellBookOpen",
		type = "game"
	},
	[6] = {
		file = "igSpellBookClose",
		type = "game"
	},
	[7] = {
		file = "TalentScreenOpen",
		type = "game"
	},
	[8] = {
		file = "igQuestLogOpen",
		type = "game"
	},
	[9] = {
		file = "Fishing Reel in",
		type = "game"
	},
	[10]= {
		file = "HumanExploration",
		type = "game"
	},
	[11]= {
		file = "applause.ogg",
		type = "addon"
	},
	[12]= {
		file = "BEEPMULT.ogg",
		type = "addon"
	},
	[13]= {
		file = "CLICK10B.ogg",
		type = "addon"
	},
	[14]= {
		file = "dogbrk.ogg",
		type = "addon"
	},
	[15]= {
		file = "pcmouseclick1.ogg",
		type = "addon"
	},
	[16]= {
		file = "stapler2.ogg",
		type = "addon"
	}
}

LFMonitor.fonts = {
	[1] = {
		file = "Fonts\\FRIZQT__.TTF",
	},
	[2] = {
		file = "Fonts\\ARIALN.TTF",
	},
	[3] = {
		file = "Fonts\\skurri.ttf",
	},
	[4] = {
		file = "Fonts\\MORPHEUS.ttf",
	}
}

LFMonitor.defaults = {
	settingsVersion = LFMonitor.settingsVersion,
	wtb = {},
	wts = {},
	lfprof = {
		professions = {},
	},
	lfw = {
		professions = {},
	},
	lfg = {
		classes = {all = true},
		roles = {},
		raids = {},
		overrideraid = false,
	},
	lfm = {
		classes = {all = true},
		roles = {},
		raids = {},
		overrideraid = false,
	},
	lfguild = {
		classes = {all = true},
		roles = {},
	},
	guildlfm = {
		classes = {all = true},
		roles = {},
	},
	monitoring = {},
	responses = {
		jc = LFMonitor.WHISPER_ME,
		enc = LFMonitor.WHISPER_ME,
		lw = LFMonitor.WHISPER_ME,
		ins = LFMonitor.WHISPER_ME,
		tlr = LFMonitor.WHISPER_ME,
		bs = LFMonitor.WHISPER_ME,
		alc = LFMonitor.WHISPER_ME,
		eng = LFMonitor.WHISPER_ME,
		lfg = LFMonitor.LFG_RESPONSE
	},
	options = {
		scl = 1,
		hinst = 1,
		rested = false,
		combat = false,
		soundon = 1,
		sound = 15,
		locked = false,
		disabled = false,
		trade = 1,
		timestamp = _G.TIMESTAMP_FORMAT_HHMMSS_24HR.." ",
		basic = "",
		adv = "",
		basicesc = "",
		useclasscolours = false,
		sensitivity = 0.5
	},
	channels = {
		general = 1,
		trade = 1,
		guild = 0
	},
	alert = {
		font = {
			index = 1,
			height = 14,
			timestamp = "ffffd200",
			player = "ff22ee55",
			text = "ffffffff"
		},
		backdrop = {
			background = "80000000",
			border = "ff87bdff"
		},
		position = {
			x = -1,
			y = -1,
			width = 575,
			height = 100
		}
	},
}

LFMonitorDB = LFMonitor:deepcopy(LFMonitor.defaults)

local fuzzyByLength = {{},{},{},{},{},{},{},{},{},{},{},{},{}}
local k, v, m, i
for k, v in pairs(LFMonitor.words.fuzzy) do
	m = string.len(k)
	for i=3,15 do
		distanceFactor = math.min(m*.5+1,i*.5+1,4.5)
		if math.abs(m-i) < distanceFactor then
			fuzzyByLength[i-2][k] = 1/distanceFactor
		end
	end
end
LFMonitor.words.fuzzyByLength = fuzzyByLength

-- Popup dialogues
StaticPopupDialogs["LFMONITOR_NOAUTOREPLY"] = {
	text = LFMonitor.CANNOT_SEND,
	button1 = _G.OKAY,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
}

StaticPopupDialogs["LFMONITOR_DEFAULTS"] = {
	text = LFMonitor.RESET_CONFIRM,
	button1 = _G.YES,
	button2 = _G.NO,
	OnAccept = function() LFMonitor:ResponseOptionsDefault() end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
}

--no operation used for certain request types
LFMonitor.nop = {""}
-- print("Modules/Initialise.lua end")
