if WoWUnit then

	-- Set up testing data for testUnlink
	local links

	GetItemInfo(19019) -- ensure item name & link will be in cache when createLinks() called

	-- BLIZZARD FIXME: Game crashes if I try to run this during login!
	-- BLIZZARD FIXME: Need a way to generate a quest/instance-lock link even when the log is empty, in case the format changes
	local function createLinks()
		local nolinks = " This message has [Square Brackets], | and : characters and the word Hitem but no links. "
		local itemName, itemLink = GetItemInfo(19019)
		local spellLink = GetSpellLink(34164)
		local spellName = GetSpellInfo(34164)
		local achLink = GetAchievementLink(1793)
		local _, achName = GetAchievementInfo(1793)
		local questLink = GetQuestLink(2) or "|cffffff00|Hquest:982:17|h[Deep Ocean, Vast Sea]|h|r"
		local questName = GetQuestLogTitle(2) or "Deep Ocean, Vast Sea"
		local instanceLink = GetSavedInstanceChatLink(1)
		local instanceName
		if instanceLink == "" then
			instanceLink = "|cffff8000|Hinstancelock:01000000003E8E2E7:531:0:15|h[Ahn'Qiraj Temple]|h|r"
			instanceName = "Ahn'Qiraj Temple"
		else
			instanceName = GetSavedInstanceInfo(1)
		end
		local _, journalInstanceName, _, _, _, _, _, journalInstanceLink = EJ_GetInstanceByIndex(1,true)
		links = {
			{
				link=nolinks,
				strict=nolinks,
				unstrict=nolinks,
			},
			{
				link=itemLink,
				strict="["..itemName.."]",
				unstrict=" Hitem "..itemName.." ",
			},
			{
				link=spellLink,
				strict="["..spellName.."]",
				unstrict=" Hspell "..spellName.." ",
			},
			{
				link=questLink,
				strict="["..questName.."]",
				unstrict=" Hquest "..questName.." ",

			},
			{
				link=achLink,
				strict="["..achName.."]",
				unstrict=" Hachievement "..achName.." ",
			},
			{
				link=journalInstanceLink,
				strict="["..journalInstanceName.."]",
				unstrict=" Hjournal "..journalInstanceName.." ",
			},
			{
				link=instanceLink,
				strict="["..instanceName.."]",
				unstrict=" Hinstancelock "..instanceName.." ",
			},
		}
	end

	-- Test suite
	local suite = {
		mocks = {
			DEFAULT_CHAT_FRAME = {
				messages = {},
				AddMessage = function(self,message,r,g,b)
					tinsert(self.messages,{message=message,r=r,g=g,b=b})
				end;
			},
		},

		testSetTestFilteringTrue = function()
			LFMonitor:setTestFiltering(true)
			-- TODO
			LFMonitor:setLogFiltering(false)
		end;

		testSetTestFilteringFalse = function()
			LFMonitor:setTestFiltering(true)
			LFMonitor:setTestFiltering(false)
			-- TODO
		end;

		testSetLogFilteringTrue = function()
			LFMonitor:setLogFiltering(true)
			-- TODO
			LFMonitor:setLogFiltering(false)
		end;

		testSetLogFilteringFalse = function()
			LFMonitor:setLogFiltering(true)
			LFMonitor:setLogFiltering(false)
			-- TODO
		end;

		testWordSimilarity = function()
			local allowedRoundingError = 1e-15
			local actual
			local function testCase(word1,word2,factor,expect,message)
				actual = LFMonitor:wordSimilarity(word1,word2,factor)
				assert(math.abs(actual - expect) <= allowedRoundingError,
				       string.format("%s\nFactor: %f\nExpected: %s\nActual: %s\n", message, factor, expect, actual))
			end
			local n
			local oneInsdel = 	{0, 1/2, 	2/3,	3/4,	4/5}
			local oneSub = 		{0, 0.5/2,	1.5/3,	2.5/4,	3.5/5}
			local twoInsdel =	{0, 0,		1/3,	2/4,	3/5}
			for n=1,5 do
				testCase("mage","mage",	1/n,1,				"identical words")
				testCase("mag","mage",	1/n,oneInsdel[n],	"letter insertion at end")
				testCase("age","mage",	1/n,oneInsdel[n],	"letter insertion at start")
				testCase("mage","magge",1/n,oneInsdel[n],	"letter insertion in mid")
				testCase("mage","mag",	1/n,oneInsdel[n],	"letter deletion at end")
				testCase("mage","age",	1/n,oneInsdel[n],	"letter deletion at start")
				testCase("mage","mge",	1/n,oneInsdel[n],	"letter deletion in mid")
				testCase("icc","ic2c",	1/n,oneInsdel[n],	"number insertion")
				testCase("aq40","aq0",	1/n,oneInsdel[n],	"number deletion")
				testCase("mage","mege",	1/n,oneSub[n],		"letter->letter substitution")
				testCase("lf1","lf2",1/n,	oneSub[n],		"number->number substitution")
				testCase("lf1","lfg",1/n,	twoInsdel[n],	"number->letter substitution")
				testCase("lfg","lf1",1/n,	twoInsdel[n],	"letter->number substitution")
				testCase("apple","orange",1/n,0,			"zero minimum")
				testCase("additive","addictive",1/n, oneInsdel[n], "long word")
			end
		end;

		testUnlink = function()
			if not links then createLinks() end
			local function testCase(input, expect, strict)
				actual = LFMonitor:unlink(input, strict)
				failure = string.format("Test failed in %s mode.\nInput: %s\nExpected output: %s\nActual output: %s\n",
					strict and "strict" or "unstrict", input, string.gsub(expect,"|","||"), string.gsub(actual,"|","||"))
				assert(actual==expect, failure)
			end
			for _, link1 in ipairs(links) do
				testCase(link1.link, link1.unstrict, false)
				testCase(link1.link, link1.strict, true)
				for _, link2 in ipairs(links) do
					for _, link3 in ipairs(links) do
						testCase(link1.link..link2.link..link3.link, link1.unstrict..link2.unstrict..link3.unstrict, false)
						testCase(link1.link..link2.link..link3.link, link1.strict..link2.strict..link3.strict, true)
					end
				end
			end
		end;

	}
	WoWUnit:AddTestSuite("LFMonitor", suite);
end
